package org.grules.config

import java.util.logging.FileHandler
import java.util.logging.Handler
import java.util.logging.Level
import java.util.logging.StreamHandler

import org.grules.functions.lib.StringFunctions
import org.grules.http.HttpRequestParametersGroup

/**
 * Default values of grules configuration properties.
 */
class DefaultGrulesConfig extends GrulesConfig {
  static final GrulesConfig INSTANCE = new DefaultGrulesConfig()
  
  static Handler createLogHandler() {
    def createDefaultHandler = {
      Exception exception ->
      new StreamHandler()
    }
    try {
      new FileHandler('gradle.log')
    } catch (IOException e) {
      createDefaultHandler(e)
    } catch (SecurityException e) {
      createDefaultHandler(e)
    } catch (GroovyRuntimeException e) {
      createDefaultHandler(e)
    }
  }

  DefaultGrulesConfig() {
    super([(GrulesConfig.NOT_VALIDATED_PARAMETERS_ACTION_PARAMETER_NAME): OnValidationEventAction.IGNORE,
    (GrulesConfig.DEFAULT_GROUP_PARAMETER_NAME): HttpRequestParametersGroup.PARAMETERS.name(),
    (GrulesConfig.LOG_LEVEL_PARAMETER_NAME): Level.FINE,
    (GrulesConfig.ENABLE_MULTITHREADING_PARAMETER_NAME): true,
    (GrulesConfig.GROUPS_PARAMETER_NAME): HttpRequestParametersGroup.values()*.name(),
    (GrulesConfig.LOGGER_HANDLER_PARAMETER_NAME): createLogHandler(),
    (GrulesConfig.RESOURCE_BUNDLE_PARAMETER_NAME): 'messages',
    (GrulesConfig.DEFAULT_FUNCTIONS_PARAMETER_NAME): [DefaultFunctionFactory.create(StringFunctions.&trim)]])
  }
}